use std::{error::Error, sync::Arc};

use common::{init_tracing, splash, util::load_or_create_config};
use config::NapGSConfig;
use data::init_data;
use logic::player::PlayerManager;
use net::NetSessionManager;

mod config;
mod database;
mod handlers;
mod logic;
mod net;

pub struct ServerState {
    pub session_mgr: NetSessionManager,
    pub player_mgr: PlayerManager,
    pub config: NapGSConfig,
}

#[tokio::main]
async fn main() -> Result<(), Box<dyn Error>> {
    splash::print("GameServer");
    init_tracing();
    tracing::info!("don't forget to visit https://discord.xeondev.com/");

    let config = load_or_create_config::<NapGSConfig>("nap_gameserver.toml");
    init_data(&config.assets).expect("failed to init data");

    let pg_pool = match common::database::init(&config.database_credentials).await {
        Ok(pool) => pool,
        Err(err) => {
            tracing::error!("failed to connect to database: {err}");
            std::process::exit(1);
        }
    };

    let state = ServerState {
        session_mgr: NetSessionManager::new(),
        player_mgr: PlayerManager::new(pg_pool),
        config,
    };

    net::listen(Arc::new(state)).await
}
